#include "../foobar2000/SDK/foobar2000.h"
#include <Shobjidl.h>

// Declaration of your component's version information
// Since foobar2000 v1.0 having at least one of these in your DLL is mandatory to let the troubleshooter tell different versions of your component apart.
// Note that it is possible to declare multiple components within one DLL, but it's strongly recommended to keep only one declaration per DLL.
// As for 1.1, the version numbers are used by the component update finder to find updates; for that to work, you must have ONLY ONE declaration per DLL. If there are multiple declarations, the component is assumed to be outdated and a version number of "0" is assumed, to overwrite the component with whatever is currently on the site assuming that it comes with proper version numbers.
DECLARE_COMPONENT_VERSION("Taskbar Playback Progress Bar", "1.1.3", "Displays progress bar for being played/paused songs with green/yellow color.\r\nCopyright (c) 2012 Alexey V. Grishin.");

// This will prevent users from renaming your component around (important for proper troubleshooter behaviors) or loading multiple instances of it.
VALIDATE_COMPONENT_FILENAME("foo_taskbar_playback_progress_bar.dll");

class taskbar_playback_progress_bar : public play_callback_static {
private:
	// hope that window will be the same all the time - as when it will got at 1st time
	HWND _wnd;
	ITaskbarList3 *_pTaskbar;
	long _progress;
	double _total_time;

	void setTaskbarStatus()
	{
		// get wnd
		if (_pTaskbar == NULL) return;
		if (this->_wnd == NULL) _wnd = core_api::get_main_window();

		// set state
		if (static_api_ptr_t<playback_control>()->is_playing()) {
			if (static_api_ptr_t<playback_control>()->is_paused()) {
				_pTaskbar->SetProgressState(_wnd, TBPF_PAUSED);
			} else {
				_pTaskbar->SetProgressState(_wnd, (_total_time < 0) ? TBPF_INDETERMINATE : TBPF_NORMAL);
			}
			_pTaskbar->SetProgressValue(_wnd, (_total_time < 0) ? 50 : _progress, 100);
		} else {
			_pTaskbar->SetProgressState(_wnd, TBPF_NOPROGRESS);
		}
	}

public:
	taskbar_playback_progress_bar()
	{
		_progress = 0;
		this->_wnd = NULL;
		_total_time = -1;
		CoInitialize(0);
		CoCreateInstance(CLSID_TaskbarList,0, CLSCTX_INPROC_SERVER,IID_ITaskbarList3,(void**)&_pTaskbar);
		if (_pTaskbar != NULL) _pTaskbar->HrInit();
	}

	void on_playback_starting(play_control::t_track_command p_command,bool p_paused) override {}

	void on_playback_new_track(metadb_handle_ptr p_track) override
	{
		_total_time = p_track->get_length();
		_progress = 0;

		setTaskbarStatus();
	}

	void on_playback_stop(play_control::t_stop_reason p_reason) override
	{
		_progress = 0;

		setTaskbarStatus();
	}

	void on_playback_seek(double p_time) override
	{
		if (_total_time > 0.0) _progress = (long)(p_time * 100 / _total_time);

		setTaskbarStatus();
	}

	void on_playback_pause(bool p_state) override
	{
		setTaskbarStatus();
	}

	void on_playback_time(double p_time) override
	{
		if (_total_time > 0.0) _progress = (long)(p_time * 100 / _total_time);

		setTaskbarStatus();
	}

	void on_playback_edited(metadb_handle_ptr p_track) override {}
	void on_playback_dynamic_info(const file_info & p_info) override {}
	void on_playback_dynamic_info_track(const file_info & p_info) override {}
	void on_volume_change(float p_new_val) override {}

	unsigned get_flags() override { return play_callback::flag_on_playback_new_track | play_callback::flag_on_playback_pause | play_callback::flag_on_playback_seek | play_callback::flag_on_playback_stop | play_callback::flag_on_playback_time; }
};

static play_callback_static_factory_t<taskbar_playback_progress_bar> foo_taskbar_playback_progress_bar;
